import React, { useRef, useEffect } from 'react';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var DragBlock = function DragBlock(props) {
  var _Children$props$class;

  var children = props.children,
      _props$horizontal = props.horizontal,
      horizontal = _props$horizontal === void 0 ? false : _props$horizontal,
      _props$vertical = props.vertical,
      vertical = _props$vertical === void 0 ? false : _props$vertical,
      disabled = props.disabled,
      defaultPosition = props.defaultPosition,
      onChange = props.onChange;
  var dragblockEl = useRef(null);
  var obj = {};
  var active = false;

  var mouseDown = function mouseDown(e) {
    obj.currentX = e.clientX;
    obj.currentY = e.clientY;
    active = true;
  };

  var mouseUp = function mouseUp(e) {
    // 如果未激活 不应该执行鼠标事件
    if (!active) return;
    active = false;
    var nodeStyle = document.defaultView.getComputedStyle(dragblockEl.current);
    var x = parseInt(nodeStyle.left);
    var y = parseInt(nodeStyle.top);
    obj.currentX = e.clientX;
    obj.currentY = e.clientY;

    if (obj.x !== x || obj.y !== y) {
      obj.x = x;
      obj.y = y;
      onChange && onChange({
        x: parseInt(obj.x),
        y: parseInt(obj.y)
      });
    }
  };

  var mouseMove = function mouseMove(e) {
    if (!active) return;
    var left = parseInt(obj.x) + e.clientX - obj.currentX;
    var top = parseInt(obj.y) + e.clientY - obj.currentY;

    if (left >= 0 && !vertical) {
      dragblockEl.current.style.left = left + 'px';
    }

    if (!horizontal) {
      dragblockEl.current.style.top = top + 'px';
    } // console.log(x, y, obj.left, e.clientX, obj.currentX);

  };

  useEffect(function () {
    if (disabled) return;
    var nodeStyle = document.defaultView.getComputedStyle(dragblockEl.current);
    var x = parseInt(nodeStyle.left);
    var y = parseInt(nodeStyle.top);
    obj.x = (defaultPosition === null || defaultPosition === void 0 ? void 0 : defaultPosition.x) || x;
    obj.y = (defaultPosition === null || defaultPosition === void 0 ? void 0 : defaultPosition.y) || y;
    window.document.addEventListener('mousemove', mouseMove);
    window.document.addEventListener('mouseup', mouseUp);
    return function () {
      window.document.removeEventListener('mousemove', mouseMove);
      window.document.removeEventListener('mouseup', mouseUp);
    };
  });
  var Children = React.Children.only(children);
  var classNameList = ((_Children$props$class = Children.props.className) === null || _Children$props$class === void 0 ? void 0 : _Children$props$class.split(' ')) || [];

  if (!disabled) {
    classNameList.push('react-dragblock');
  }

  return React.cloneElement(Children, {
    className: classNameList.join(' '),
    style: _objectSpread2({}, Children.props.style, {
      cursor: 'move',
      position: 'absolute',
      left: "".concat(defaultPosition === null || defaultPosition === void 0 ? void 0 : defaultPosition.x, "px"),
      top: "".concat(defaultPosition === null || defaultPosition === void 0 ? void 0 : defaultPosition.y, "px")
    }),
    ref: dragblockEl,
    onMouseDown: mouseDown
  }); // return (
  //   <div
  //     ref={dragblockEl}
  //     className="react-dragblock"
  //     onMouseDown={mouseDown}
  //     style={{
  //       left: `${defaultPosition?.x}px`,
  //       top: `${defaultPosition?.y}px`,
  //     }}
  //   >{children}</div>
  // );
};

export default DragBlock;
